from azure.identity import DefaultAzureCredential
from icloudservice.src.service.azure.azure_services import AzureService
from icloudservice.src.service.azure.azure_services import BlobStorageService, CosmosDBService, ServiceBusService


class AzureCloud(AzureService):
    def __init__(self, subscription_id: str, credential: DefaultAzureCredential = None):
        self.subscription_id = subscription_id
        self.credential = credential or DefaultAzureCredential()
        super().__init__(subscription_id=subscription_id)

    def BlobStorageService(self, container_name: str):
        return BlobStorageService(self, container_name)

    def CosmosDBService(self, database_name: str):
        return CosmosDBService(self, database_name)

    def ServiceBusService(self):
        return ServiceBusService(self)


from azure.storage.blob import BlobServiceClient
from azure.servicebus import ServiceBusClient, ServiceBusMessage
from azure.identity import DefaultAzureCredential
from azure.cosmos import CosmosClient
from rich.progress import Progress


class AzureService:

    def __init__(self, subscription_id: str, credential: DefaultAzureCredential = None):
        self.subscription_id = subscription_id
        self.credential = credential or DefaultAzureCredential()
        
        # Configura los endpoints para los servicios específicos
        self.blob_endpoint = f"https://{subscription_id}.blob.core.windows.net"
        self.cosmos_endpoint = f"https://{subscription_id}.documents.azure.com:443/"
        self.service_bus_endpoint = f"https://{subscription_id}.servicebus.windows.net/"

    def create_blob_client(self):
        if self.blob_endpoint:
            return BlobServiceClient(account_url=self.blob_endpoint, credential=self.credential)
        else:
            raise ValueError("Blob endpoint is not configured")

    def create_cosmos_client(self):
        if self.cosmos_endpoint:
            return CosmosClient(endpoint=self.cosmos_endpoint, credential=self.credential)
        else:
            raise ValueError("Cosmos endpoint is not configured")
    def create_service_bus_client(self):
        if self.service_bus_endpoint:
            return ServiceBusClient(fully_qualified_namespace=self.service_bus_endpoint, credential=self.credential)
        else:
            raise ValueError("Service Bus endpoint is not configured")

class BlobStorageService:
    def __init__(self, azure_service: AzureService, container_name: str):
        self.client = azure_service.create_blob_client()
        self.container_name = container_name

    def upload(self, local_path: str, blob_name: str):
        blob_client = self.client.get_blob_client(container=self.container_name, blob=blob_name)
        with open(local_path, "rb") as data:
            blob_client.upload_blob(data)

    def download(self, blob_name: str, local_path: str):
        blob_client = self.client.get_blob_client(container=self.container_name, blob=blob_name)
        with open(local_path, "wb") as data:
            data.write(blob_client.download_blob().readall())

    def list_blobs(self):
        container_client = self.client.get_container_client(self.container_name)
        blobs = list(container_client.list_blobs())
        total_blobs = len(blobs)
        with Progress() as progress:
            task = progress.add_task("[cyan]Listing blobs...", total=total_blobs)
            blob_names = []
            for blob in blobs:
                blob_names.append(blob.name)
                progress.update(task, advance=1)

        return blob_names

class CosmosDBService:
    def __init__(self, azure_service: AzureService, database_name: str):
        self.client = azure_service.create_cosmos_client()
        self.database = self.client.get_database_client(database_name)

    def create_container(self, container_name: str):
        self.database.create_container(id=container_name)

    def query_items(self, container_name: str, query: str):
        container = self.database.get_container_client(container_name)
        return list(container.query_items(query=query, enable_cross_partition_query=True))

class ServiceBusService:
    def __init__(self, azure_service: AzureService):
        self.client = azure_service.create_service_bus_client()

    def send_message(self, queue_name: str, message_body: str):
        sender = self.client.get_queue_sender(queue_name)
        with sender:
            message = ServiceBusMessage(message_body)
            sender.send_messages(message)

    def receive_messages(self, queue_name: str):
        receiver = self.client.get_queue_receiver(queue_name)
        with receiver:
            messages = receiver.receive_messages()
            return [msg.body for msg in messages]


import json
import boto3

class AWSService:
    def __init__(self, access_key: str, secret_key: str):
        self.access_key = access_key
        self.secret_key = secret_key

    def create_client(self, service_name: str):
        return boto3.client(
            service_name,
            aws_access_key_id=self.access_key,
            aws_secret_access_key=self.secret_key
        )

class S3Service():
    def __init__(self, aws_service: AWSService, bucket_name: str):
        self.client = aws_service.create_client('s3')
        self.bucket_name = bucket_name

    def upload(self, local_path: str, remote_path: str):
        self.client.upload_file(local_path, self.bucket_name, remote_path)

    def download(self, remote_path: str, local_path: str):
        self.client.download_file(self.bucket_name, remote_path, local_path)

    def list_files(self, directory: str):
        response = self.client.list_objects_v2(Bucket=self.bucket_name, Prefix=directory)
        return [obj['Key'] for obj in response.get('Contents', [])]

    def list_buckets(self):
        response = self.client.list_buckets()
        return [bucket['Name'] for bucket in response.get('Buckets', [])]

class LambdaService:
    def __init__(self, aws_service: AWSService):
        self.client = aws_service.create_client('lambda')

    def invoke(self, function_name: str, payload: dict):
        response = self.client.invoke(FunctionName=function_name, Payload=json.dumps(payload))
        return response['Payload'].read().decode()

class EC2Service:
    def __init__(self, aws_service: AWSService):
        self.client = aws_service.create_client('ec2')

    def start_instance(self, instance_id: str):
        self.client.start_instances(InstanceIds=[instance_id])

    def stop_instance(self, instance_id: str):
        self.client.stop_instances(InstanceIds=[instance_id])

    def list_instances(self):
        response = self.client.describe_instances()
        instances = []
        for reservation in response['Reservations']:
            for instance in reservation['Instances']:
                instances.append(instance['InstanceId'])
        return instances

class SQSService:
    def __init__(self, aws_service: AWSService):
        self.client = aws_service.create_client('sqs')

    def send_message(self, queue_url: str, message_body: str):
        self.client.send_message(QueueUrl=queue_url, MessageBody=message_body)

    def receive_message(self, queue_url: str):
        response = self.client.receive_message(QueueUrl=queue_url)
        return response.get('Messages', [])

class SNSService:
    def __init__(self, aws_service: AWSService):
        self.client = aws_service.create_client('sns')

    def publish_message(self, topic_arn: str, message: str):
        self.client.publish(TopicArn=topic_arn, Message=message)

    def list_topics(self):
        response = self.client.list_topics()
        return [topic['TopicArn'] for topic in response.get('Topics', [])]

class AWSCloud(AWSService):
    def __init__(self, access_key: str, secret_key: str):
        super().__init__(access_key, secret_key)

    def S3Service(self, bucket_name: str):
        return S3Service(self, bucket_name)

    def LambdaService(self):
        return LambdaService(self)

    def EC2Service(self):
        return EC2Service(self)

    def SQSService(self):
        return SQSService(self)

    def SNSService(self):
        return SNSService(self)
